-- author RivalBomb

claasLexion430 = {};

function claasLexion430.prerequisitesPresent(specializations)
	print("......LOADING CLAAS LEXION 400 by RIVALBOMB......");
    return SpecializationUtil.hasSpecialization(Motorized, specializations) and SpecializationUtil.hasSpecialization(Overloading, specializations);
end;

function claasLexion430:load(xmlFile)
	self.playerCallback = SpecializationUtil.callSpecializationsFunction("playerCallback");

	self.isStrawEnabled = false;
	self:setIsStrawEnabled(false, true, true);

	self.displayLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lexion430Animations.displayLight#index"));
	if g_rearMirrorsAvailable then	-- show display light only on high or very high graphic settings
		setVisibility(self.displayLight, true);
	else
		setVisibility(self.displayLight, false);
	end;
	
	if self.isClient then
		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.unloadingSound#node"));
		self.unloadingSound = Utils.loadSample(xmlFile, {}, "vehicle.unloadingSound", nil, self.baseDirectory, node);
		local node2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.strawChopperSound#node"));
		self.strawChopperSound = Utils.loadSample(xmlFile, {}, "vehicle.strawChopperSound", nil, self.baseDirectory, node2);
		local node3 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.maizeChopperSound#node"));
		self.maizeChopperSound = Utils.loadSample(xmlFile, {}, "vehicle.maizeChopperSound", nil, self.baseDirectory, node3);
	end;

	-- Blinkerhebel
	self.blinkerHebel = {};
	self.blinkerHebel.index = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lexion430Animations.Blinkerhebel#index"))
	self.blinkerHebel.rotLeft = getXMLString(xmlFile, "vehicle.lexion430Animations.Blinkerhebel#rotLeft");
	self.blinkerHebel.rotRight = getXMLString(xmlFile, "vehicle.lexion430Animations.Blinkerhebel#rotRight");
	self.blinkerHebel.rotNormal = getXMLString(xmlFile, "vehicle.lexion430Animations.Blinkerhebel#rotNormal");
	
	
	-- Stroh-Haufen
	self.strawBunch = {};
	
	local i = 0;
	while true do
		local key = string.format("vehicle.strawBunches.strawBunch(%d)",i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local node = getXMLString(xmlFile, key.. "#index");
		if node == nil then
			break;
		end;
		local trigger = Utils.indexToObject(self.components, getXMLString(xmlFile, key.. "#trigger"));
		if trigger == nil then
			break;
		end;

		i = i + 1;
		self.strawBunch[i] = {};
		self.strawBunch[i].index = Utils.indexToObject(self.components, node);
		self.strawBunch[i].maxYtrans = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#maxScale"), 1);
		self.strawBunch[i].resetOnChopperSwitch = Utils.getNoNil(getXMLBool(xmlFile, key .. "#resetOnChopperSwitch"), false);
		self.strawBunch[i].curTrans = 0;
		self.strawBunch[i].trigger = trigger;
		self.strawBunch[i].plIR = false;
		
		addTrigger( trigger, "playerCallback", self );
	end;
end;
function claasLexion430:delete()
	if self.unloadingSound ~= nil then
		Utils.deleteSample(self.unloadingSound);
	end;
	if self.strawChopperSound ~= nil then
		Utils.deleteSample(self.strawChopperSound);
	end;
	if self.maizeChopperSound ~= nil then
		Utils.deleteSample(self.maizeChopperSound);
	end;
	for i,j in pairs(self.strawBunch) do
		if j.trigger ~= nil then
			removeTrigger(j.trigger);
		end;
	end;
end;
function claasLexion430:readStream(streamId, connection)
	for i=1, #self.strawBunch do
		self.strawBunch[i].curTrans = streamReadFloat32(streamId);
	end;
end;
function claasLexion430:writeStream(streamId, connection)
	for i=1, #self.strawBunch do
		streamWriteFloat32(streamId, self.strawBunch[i].curTrans);
	end;
end;
function claasLexion430:mouseEvent(posX, posY, isDown, isUp, button)
end;
function claasLexion430:keyEvent(unicode, sym, modifier, isDown)
end;
function claasLexion430:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		for i,j in pairs(self.strawBunch) do
			local s = Utils.getNoNil( getXMLFloat(xmlFile, key..string.format("#strawBunchScale%d",i) ), 0);
			self.strawBunch[i].curTrans = s;
		end;
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function claasLexion430:getSaveAttributesAndNodes(nodeIdent)
    local attributes = '';
	for i,j in pairs(self.strawBunch) do
		attributes = attributes .. string.format('strawBunchScale%d="%s" ', i, tostring(j.curTrans));
	end;
    return attributes, nil;
end;
function claasLexion430:update(dt)
	
	if self.isEntered and self:getIsActiveForInput(false) then
		if self.camIndex == 2 and InputBinding.hasEvent(InputBinding.resetCamRotCL430) then
			self.cameras[2].rotX = -25*(math.pi/180);
			self.cameras[2].rotY = math.pi;
			self.cameras[2].rotZ = 0;
		end;
	end;
    if self:getIsActive() then
        if self.isClient and self:getIsActiveForInput(false) and not self:hasInputConflictWithSelection() then
            if InputBinding.hasEvent(InputBinding.EMPTY_GRAIN2) then
                if self.hasPipe then
                    local nextState = self.targetPipeState+1;
                    if nextState > self.numPipeStates then
                        nextState = 1;
                    end;
                    if self:getIsPipeStateChangeAllowed(nextState) then
                        self:setPipeState(nextState);
                    elseif nextState ~= 1 and self:getIsPipeStateChangeAllowed(1) then
                        -- also try to close the pipe if other states are not allowed
                        self:setPipeState(1);
                    end
                else
                    self:setOverloadingActive(not self.isOverloadingActive);
                end;
            end;
        end;
		
		
			
		for i=1, #self.strawBunch do
			if self.chopperPSenabled or self.strawPSenabled then
				if self.strawBunch[i].resetOnChopperSwitch then
					if not self.isStrawEnabled then
						self.strawBunch[i].curTrans = math.min(self.strawBunch[i].curTrans+0.000005, self.strawBunch[i].maxYtrans);
					end;
				else
					self.strawBunch[i].curTrans = math.min(self.strawBunch[i].curTrans+0.000005, self.strawBunch[i].maxYtrans);
				end;
			end;
			if self.isStrawEnabled and self.strawBunch[i].resetOnChopperSwitch then
				self.strawBunch[i].curTrans = 0;
			end;
		end;
	end;
	local plIR = false;
	local id = 0;
	for i,j in pairs(self.strawBunch) do
		if j.plIR then
			id = i;
			plIR = true;
			break;
		end;
	end;
	if plIR then
		g_currentMission:addHelpButtonText( g_i18n:getText("CLEAR_STRAW_BUNCH"), InputBinding.CLEAR_STRAW_BUNCH);
		if InputBinding.hasEvent(InputBinding.CLEAR_STRAW_BUNCH) then
			self.strawBunch[id].curTrans = 0;
		end;
	end;
	for i=1, #self.strawBunch do
		setScale(self.strawBunch[i].index, self.strawBunch[i].curTrans, self.strawBunch[i].curTrans, math.min(self.strawBunch[i].curTrans*2, self.strawBunch[i].maxYtrans));
	end;
	if self.isClient and self.unloadingSound.sound3D ~= nil then
		if self.pipeIsUnloading then
			if not self.unloadingSoundPlaying then
				setVisibility(self.unloadingSound.sound3D, true);
				self.unloadingSoundPlaying = true;
			end;
		else
			if self.unloadingSoundPlaying then
				setVisibility(self.unloadingSound.sound3D, false);
				self.unloadingSoundPlaying = false;
			end;
		end;
	end;
	if self.isClient and self.strawChopperSound.sound3D ~= nil and self.maizeChopperSound.sound3D ~= nil then
		if self.chopperPSenabled then
			if self.lastValidFillType == Fillable.FILLTYPE_MAIZE then
				setVisibility(self.maizeChopperSound.sound3D, true);
			else
				setVisibility(self.strawChopperSound.sound3D, true);
			end;
		else
			setVisibility(self.strawChopperSound.sound3D, false);
			setVisibility(self.maizeChopperSound.sound3D, false);
		end;
	end;
end;
function claasLexion430:updateTick(dt)
	if self.turnSignalState~=nil then
		if self.turnSignalState==1 then	-- Blinker links
			setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotLeft));
		elseif self.turnSignalState==2 then	-- Blinker rechts
			setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotRight));
		else -- kein Blinker aktiv / Warnblinker
			setRotation(self.blinkerHebel.index, 0, 0, math.rad(self.blinkerHebel.rotNormal));
		end;
    end;
end;
function claasLexion430:draw()
end;
function claasLexion430:playerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	local id = 0;
	for i,j in pairs(self.strawBunch) do
		if j.trigger == triggerId then
			id = i;
			break;
		end;
	end;
	
	if id ~= 0 then
		if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.strawBunch[id].plIR = true;
		elseif onLeave then
			self.strawBunch[id].plIR = false;
		end;
	end
	
end;
